

{-----------------------------------------------------------------------------
 Unit Name: ParalTransport
 Project Name : Superficies
 Author:    Angel Montesinos Amilibia
 05/01/02 
 Purpose: A special Runge-Kutta algorithm for computing the parallel trans-
  port along a polygonal line such as the line traced by the user with the
  mouse as registered by the operating system.
-----------------------------------------------------------------------------}
{*-
 * Copyright (c) 2002 Angel Montesinos Amilibia
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *        Version 4.0.0.0
 *}



unit ParalTransport;
interface
uses
  Windows,
  Globals;

procedure InitArrow(x, y: Integer);
function ParallelTransportArrow(lastPt, curPt: TPoint): Boolean;
procedure StoreFinalArrow(pt: TPoint);

implementation
uses
  Math, Graphics,
  FSurface, Utilities, BasicRiemUtils, ParametricUtils;

var
  tMax: double;
  theArrow: vector2;
  arrowFoot, arrowPoint: TPoint;

procedure DrawArrow(penMode: TPenMode);
begin
  with surface.Drawing2D.canvas do begin
    Pen.Mode := penMode;
    Pen.Color := gLinesData[kArrows].pen2DColor;
    MoveTo(arrowFoot.X, arrowFoot.y);
    LineTo(arrowPoint.x, arrowPoint.y);
  end;
end;

procedure InitArrow(x, y: Integer);
var
  origin, endPt: vector2;
begin
  theArrow[1] := dp1 * kPixelsArrow;
  theArrow[2] := 0;
  arrowFoot.x := x;
  arrowFoot.y := y;
  arrowPoint.x := x + kPixelsArrow;
  arrowPoint.y := y;
  DrawArrow(pmCopy);
  DrawArrow(pmNotXoR);
  origin[1] := arrowFoot.X;
  origin[2] := arrowFoot.Y;
  endPt[1] := arrowPoint.x;
  endPt[2] := arrowPoint.y;
  gActiveLine := kArrows;
  AddSegment(origin, endPt, 2, 0);
  gActiveLine := kTransport;
end;

procedure StoreFinalArrow(pt: TPoint);
var
  origin, endPt: vector2;
  paramOrigin: vector2;
begin
  FromDrawingToParameters(pt, paramOrigin);
  endPt[1] := paramOrigin[1] + theArrow[1];
  endPt[2] := paramOrigin[2] + theArrow[2];
  FromParametersToDrawing(endPt, arrowPoint);
  gActiveLine := kArrows;
  origin[1] := pt.X;
  origin[2] := pt.Y;
  endPt[1] := arrowPoint.x;
  endPt[2] := arrowPoint.y;
  AddSegment(origin, endPt, 2, 0);
  gActiveLine := kTransport;
  arrowFoot := pt;
end;

function RunKut(var step, estimStep, t: double; var arrow: vector2;
  initialX, initialVel: vector2): boolean;
const
  tol: double = 1.0E-5;
var
  err, coef: double;
  goOn: boolean;
  i: integer;
  rk1, rk2, rk3: vector2;

  function ComputeDerivative(var deriv: vector2; t: double;
    curArrow: vector2): boolean;
  var
    i: integer;
    place: vector3;
    metric, gammaX, gammaY, factors: Vector3;
  begin
    ComputeDerivative := FALSE;
    for i := 1 to 2 do
      place[i] := initialX[i] + t * initialVel[i];
    if gType = PseudoRiemannian then
      goOn := ComputeChristSymbPseudoRiem(place, metric, gammaX, gammaY)
    else
      goOn := ComputeChristSymbParam(place, metric, gammaX, gammaY);
    if goOn then begin
      ComputeDerivative := TRUE;
      factors[1] := initialVel[1] * curArrow[1];
      factors[2] := initialVel[1] * curArrow[2] + initialVel[2] *
        curArrow[1];
      factors[3] := initialVel[2] * curArrow[2];
      deriv[1] := -(gammaX[1] * factors[1] + gammaX[2] * factors[2] +
        gammaX[3] * factors[3]);
      deriv[2] := -(gammaY[1] * factors[1] + gammaY[2] * factors[2] +
        gammaY[3] * factors[3]);
    end;
  end;

  function CalcElem(var error: double; var arrow: vector2;
    t, guessStep: double): boolean;
  var
    fl: vector2;
    goOn: boolean;
    i: integer;
  begin
    fl := arrow;
    goOn := ComputeDerivative(rk1, t, fl);
    if goOn then begin
      for i := 1 to 2 do
        fl[i] := arrow[i] + guessStep * rk1[i];
      goOn := ComputeDerivative(rk2, t + guessStep, fl);
      if goOn then begin
        for i := 1 to 2 do
          fl[i] := arrow[i] + 0.25 * guessStep * (rk1[i] + rk2[i]);
        goOn := ComputeDerivative(rk3, t + 0.5 * guessStep, fl);
        if goOn then begin
          error := guessStep * hypot(2.0 * rk3[1] - rk1[1] - rk2[1], 2.0 *
            rk3[2] - rk1[2] - rk2[2]);
          error := (error + tol * 0.001) / gSpan2D;
        end;
      end;
    end;
    CalcElem := goOn;
  end;

begin
  repeat
    goOn := CalcElem(err, arrow, t, estimStep);
    if goOn then begin
      coef := 0.8 * exp(ln(tol / err) / 3.0);
      if coef < 0.5 then
        coef := 0.5;
      if 2.0 < coef then
        coef := 2.0;
      step := estimStep;
      estimStep := coef * step;
    end
  until (not goOn) or (err < tol);
  if goOn then begin
    for i := 1 to 2 do
      arrow[i] := arrow[i] + step * (rk1[i] + rk2[i] + 4.0 * rk3[i]) / 6;
    t := t + step;
    if (t + estimStep > tMax) then
      estimStep := tMax - t;
  end;
  RunKut := goOn;
end;

function ParallelTransportArrow(lastPt, curPt: TPoint): Boolean;
var
  initialX, finalX: Vector2;
  iniTPt, finTPt: Vector2;
  aux: single;
  can: TCanvas;
  function ComputeArrow(initialPoint, curPt: TPoint; var theArrow:
    vector2)
      : Boolean;
  var
    vel, arr: vector2;
    x: Vector3;
    t, step, estimStep: double;
    i: integer;
  begin
    Result := PtInRect(gRect, initialPoint) and PtInRect(gRect,
      curPt);
    if not Result then
      Exit;
    FromDrawingToParameters(initialPoint, initialX);
    FromDrawingToParameters(curPt, finalX);

    if gType = Parametric then
      result := InRangeAndClips(x, finalX)
    else
      result := min(gHalfEdges2D[1] - abs(finalX[1] - gCenter2D[1]),
        gHalfEdges2D[2] - abs(finalX[2] - gCenter2D[2])) >= 0;
    if not result then
      Exit;
    tMax := hypot(finalX[1] - initialX[1], finalX[2] - initialX[2]) /
      gRadius2D;
    if tMax < 1.0E-9 then
      Exit;
    for i := 1 to 2 do
      vel[i] := (finalX[i] - initialX[i]) / tMax;
    step := 0.1 * tMax;
    estimStep := step;
    arr := theArrow;
    t := 0;
    repeat
      Result := RunKut(step, estimStep, t, arr, initialX, vel);
    until (not Result) or (t + 0.0001 > tMax) or (step < 0.1 * gRadius2D);
    if Result then
      theArrow := arr;
  end;

begin
  Result := ComputeArrow(lastPt, curPt, theArrow);
  if not Result then
    Exit;
  gActiveLine := kTransport;
  DrawArrow(pmNotXor);
  can := surface.Drawing2D.canvas;
  can.Pen.Mode := pmCopy;
  can.Pen.Color := gLinesData[kTransport].pen2DColor;
  can.moveto(lastPt.x, lastPt.y);
  can.lineto(curPt.x, curPt.y);
  iniTPt[1] := lastPt.X;
  initPt[2] := lastPt.Y;
  finTPt[1] := curPt.X;
  finTPt[2] := curPt.Y;
  AddSegment(iniTPt, finTPt, 2, 0);
  aux := max(abs(theArrow[1]) / gEdges2D[1], abs(theArrow[2]) /
    gEdges2D[2]);
  if aux > 1.415 then
    aux := 1.415 / aux
  else
    aux := 1;
  arrowPoint.x := curPt.x + round(aux * theArrow[1] * kWidth /
    gEdges2D[1]);
  arrowPoint.y := curPt.y - round(aux * theArrow[2] * kHeight /
    gEdges2D[2]);
  arrowFoot := curPt;
  DrawArrow(pmNotXor);
end;

end.
