

{-----------------------------------------------------------------------------
 Unit Name: ReadForms
 Project Name : Superficies
 Author:    Angel Montesinos Amilibia
 05/01/02 
 Purpose: Routines for registering and verifying the data entered by the user
    for defining a surface.
-----------------------------------------------------------------------------}
{*-
 * Copyright (c) 2002 Angel Montesinos Amilibia
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *        Version 4.0.0.0
 *}



unit ReadForms;

interface

function ReadForm: Integer;

implementation

uses
  Globals, FSurface, OcFunct, ResStrings, SysUtils, Classes;

type
  EReadFormError = class(Exception)
    ErrorCode: Integer;
  end;

var
  newF1, newF2, newF3, newF4: TOpCodeFunction;
  uMin, uMax, vMin, vMax, wMIn, wMax: Double;
  grid: Integer;
  readStrings: array[1..kNumOfSavedToDiskStrings] of AnsiString;

procedure createFunctions;
begin
  newF1 := TOpCodeFunction.create;
  newF2 := TOpCodeFunction.create;
  newF3 := TOpCodeFunction.create;
  newF4 := TOpCodeFunction.create;
end;

procedure freeFunctions;
begin
  newF1.free;
  newF2.free;
  newF3.free;
  newF4.free;
end;

procedure AssignFunctions;
begin
  gTheF1.Free;
  gTheF1 := newF1;
  if gType <> Implicit then begin
    gTheF2.Free;
    gTheF2 := newF2;
    gTheF3.Free;
    gTheF3 := newF3;
  end;
  if gType = Tube then begin
    gTheF4.Free;
    gTheF4 := newF4;
  end;
end;



function ReadFormParametric: Integer;
var
  warning, vars: AnsiString;
  separator : char;

  procedure HandleFormReadingException(mess: AnsiString);
  begin
    ReadFormParametric := 2;
    surface.Inform(mess);
    beep;
  end;

begin
  if gType = Parametric then
    vars := 'uv'
  else
    vars := 'xy';
  result := 0;
  try
    readStrings[1] := Trim(surface.f1Edit.Text);
    warning := newF1.ParseNewText(readStrings[1], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[1] <> gSurfaceStr[1] then
      Result := 1;

    readStrings[2] := Trim(surface.f2Edit.Text);
    warning := newF2.ParseNewText(readStrings[2], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[2] <> gSurfaceStr[2] then
      Result := 1;

    readStrings[3] := Trim(surface.f3Edit.Text);
    warning := newF3.ParseNewText(readStrings[3], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[3] <> gSurfaceStr[3] then
      Result := 1;

    readStrings[5] := Trim(surface.uMinEdit.Text);
    warning := ReadConstantField(readStrings[5], uMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[5] <> gSurfaceStr[5] then
      Result := 1;

    readStrings[6] := Trim(surface.uMaxEdit.Text);
    warning := ReadConstantField(readStrings[6], uMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[6] <> gSurfaceStr[6] then
      Result := 1;

    readStrings[7] := Trim(surface.vMinEdit.Text);
    warning := ReadConstantField(readStrings[7], vMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[7] <> gSurfaceStr[7] then
      Result := 1;

    readStrings[8] := Trim(surface.vMaxEdit.Text);
    warning := ReadConstantField(readStrings[8], vMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[8] <> gSurfaceStr[8] then
      Result := 1;
    if gType = Parametric then begin
      try
        separator := DecimalSeparator;
        DecimalSeparator := '.';
        grid := Trunc(Abs(StrToFloat(Trim(surface.wMaxEdit.Text))));
        DecimalSeparator := separator;
      except
        on EConvertError do begin
          grid := 60;
          DecimalSeparator := separator;
        end;
      end;
      if (grid < 10) then grid := 10;
      if (grid > 127) then grid := 127;
      surface.wMaxEdit.Text := IntToStr(grid);
      readStrings[10] := surface.wMaxEdit.Text;
      if readStrings[10] <> gSurfaceStr[10] then
        Result := 1;
    end;

    if (uMin >= uMax) then begin
      warning := eUMinLessThanUMax;
      raise EReadFormError.Create(warning);
    end;
    if (vMin >= vMax) then begin
      warning := eVMinLessThanVMax;
      raise EReadFormError.Create(warning);
    end;
    gMin2D[1] := uMin;
    gMin2D[2] := vMin;
    gMax2D[1] := uMax;
    gMax2D[2] := vMax;
    if gType = Parametric then begin
      gGridU := grid;
      gGridV := grid;
    end;
  except
    on EReadFormError do
      HandleFormReadingException(warning);
  end;
end;

function ReadFormImplicit: Integer;
var
  warning, vars: AnsiString;

  procedure HandleFormReadingException(mess: AnsiString);
  begin
    ReadFormImplicit := 2;
    surface.Inform(mess);
    beep;
  end;

begin
  vars := 'xyz';
  result := 0;
  try
    readStrings[1] := Trim(surface.f1Edit.Text);
    warning := newF1.ParseNewText(readStrings[1], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[1] <> gSurfaceStr[1] then
      Result := 1;

    readStrings[5] := Trim(surface.uMinEdit.Text);
    warning := ReadConstantField(readStrings[5], uMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[5] <> gSurfaceStr[5] then
      Result := 1;

    readStrings[6] := Trim(surface.uMaxEdit.Text);
    warning := ReadConstantField(readStrings[6], uMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[6] <> gSurfaceStr[6] then
      Result := 1;

    readStrings[7] := Trim(surface.vMinEdit.Text);
    warning := ReadConstantField(readStrings[7], vMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[7] <> gSurfaceStr[7] then
      Result := 1;

    readStrings[8] := Trim(surface.vMaxEdit.Text);
    warning := ReadConstantField(readStrings[8], vMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[8] <> gSurfaceStr[8] then
      Result := 1;

    readStrings[9] := Trim(surface.wMinEdit.Text);
    warning := ReadConstantField(readStrings[9], wMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[9] <> gSurfaceStr[9] then
      Result := 1;

    readStrings[10] := Trim(surface.wMaxEdit.Text);
    warning := ReadConstantField(readStrings[10], wMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[10] <> gSurfaceStr[10] then
      Result := 1;

    if (uMin >= uMax) then begin
      warning := eXMinLessThanXMax;
      raise EReadFormError.Create(warning);
    end;
    if (vMin >= vMax) then begin
      warning := eYMinLessThanYMax;
      raise EReadFormError.Create(warning);
    end;
    if (wMin >= wMax) then begin
      warning := eZMinLessThanZMax;
      raise EReadFormError.Create(warning);
    end;
    gMin3D[1] := uMin;
    gMin3D[2] := vMin;
    gMin3D[3] := wMin;
    gMax3D[1] := uMax;
    gMax3D[2] := vMax;
    gMax3D[3] := wMax;
  except
    on EReadFormError do
      HandleFormReadingException(warning);
  end;
end;

function ReadFormTube: Integer;
var
  warning, vars: AnsiString;

  procedure HandleFormReadingException(mess: AnsiString);
  begin
    ReadFormTube := 2;
    surface.Inform(mess);
    beep;
  end;

begin
  vars := 't';
  result := 0;
  try
    readStrings[1] := Trim(surface.f1Edit.Text);
    warning := newF1.ParseNewText(readStrings[1], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[1] <> gSurfaceStr[1] then
      Result := 1;

    readStrings[2] := Trim(surface.f2Edit.Text);
    warning := newF2.ParseNewText(readStrings[2], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[2] <> gSurfaceStr[2] then
      Result := 1;

    readStrings[3] := Trim(surface.f3Edit.Text);
    warning := newF3.ParseNewText(readStrings[3], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[3] <> gSurfaceStr[3] then
      Result := 1;

    readStrings[4] := Trim(surface.f4Edit.Text);
    warning := newF4.ParseNewText(readStrings[4], vars);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[4] <> gSurfaceStr[4] then
      Result := 1;

    readStrings[5] := Trim(surface.uMinEdit.Text);
    warning := ReadConstantField(readStrings[5], uMin);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[5] <> gSurfaceStr[5] then
      Result := 1;

    readStrings[6] := Trim(surface.uMaxEdit.Text);
    warning := ReadConstantField(readStrings[6], uMax);
    if warning <> '' then
      raise EReadFormError.Create(warning);
    if readStrings[6] <> gSurfaceStr[6] then
      Result := 1;

    if (uMin >= uMax) then begin
      warning := eUMinLessThanUMax;
      raise EReadFormError.Create(warning);
    end;

    gMin2D[1] := uMin;
    gMin2D[2] := 0;
    gMax2D[1] := uMax;
    gMax2D[2] := 2 * pi;
    gGridU := kTubeGridU;
    gGridV := kTubeGridV;
  except
    on EReadFormError do
      HandleFormReadingException(warning);
  end;
end;

function ReadForm: Integer;
var
  i: Integer;
begin
  createFunctions;
  surface.Inform('');
  for i := 1 to kNumOfSavedToDiskStrings do
    readStrings[i] := '';
  case gType of
    Parametric, PseudoRiemannian:
      result := ReadFormParametric;
    Implicit:
      result := ReadFormImplicit;
    Tube:
      result := ReadFormTube;
  else
    ;
  end;
  if result < 2 then begin
    AssignFunctions;
    for i := 1 to kNumOfSavedToDiskStrings do begin
      gSurfaceStr[i] := readStrings[i];
      gSurfaceStrLengths[i] := Length(readStrings[i]);
    end;
  end
  else
    freeFunctions;
end;

end.
